;; Claw
;; Copyright 2017 - Michael Kohn
;;
;; P50 is output to servo
;; PD70-77 is debug to LEDs
;;
;; Timer 2 is prescaled / 16
;; 3686400 MHz / 16 = 230400 Hz = 4.3uS
;; 230400 interrupts/s * 0.020s = 4608 interrupts
;; 4608 interuppts / 20 = 230.4 interrupts (1ms)
;; 230 interrupts / 20 = 11.5 (12) interrupts (should give 20 different values)
;; 50us per interrupt
;; 20 interrupts = 1ms, 40 interrupts = 2ms, 400 interrupts = 20ms
;; 20,000 interrupts = 1 second
;;

.65816

.include "w65c265.inc"

.define TOP_SERVO_COUNTER 400

.define HIPPO_REST_0 360
.define HIPPO_REST_1 360
.define HIPPO_REST_2 360
.define HIPPO_REST_3 360

.define HIPPO_PULL_0 380
.define HIPPO_PULL_1 380
.define HIPPO_PULL_2 380
.define HIPPO_PULL_3 380

.macro CHECK_SERVO(servo_count,mask)
.scope
  cpy servo_count
  bne timer2_do_not_update_servo
  lda PD5
  and.b #mask
  sta PD5
timer2_do_not_update_servo:
.ends
.endm

.macro SET_SERVO(servo_count,value,servo_interrupt)
  ldx #value
  stx servo_count
  ldx #20000
  stx servo_interrupt
.endm

.macro SET_SERVO_REST(servo_count,value)
  ldx #value
  stx servo_count
.endm

.macro CLEAR_UART_IF
  lda.b #0x00
  sta UIFR
.endm

.org 0xf0
servo_count_0:
  resw 1
servo_count_1:
  resw 1
servo_count_2:
  resw 1
servo_count_3:
  resw 1
interrupt_count_0:
  resw 1
interrupt_count_1:
  resw 1
interrupt_count_2:
  resw 1
interrupt_count_3:
  resw 1
interrupt_count:
  resw 1
.org 0xc0
delay:
  dw TOP_SERVO_COUNTER

.org 0x1000
start:
  ; Disable interrupts
  sei

  ; Set native mode
  clc
  xce

  ; Set A to 8-bit
  sep #0x20

  ; Set X/Y to 16-bit
  rep #0x10

  ; Set port 7 in data mode.  Port 7 seems to be always an output
  ; port.  The PCS7 register needs to be set to 0 to let this port
  ; work.
  ;lda.b #0x00
  ;sta PDD7

  ; Set port 7 value to 0x80.  Port 7 is hooked up to 8 LEDs.  Every
  ; bit in the PD7 register is conneted to one LED.  So by setting this
  ; register to 0x80 (binary 10000000), LED number 7 will be turned on.
  lda.b #0x00
  sta PD7

  ; Timer control
  ; Bit 7: UART3 timer (0 = Timer 3, 1 = Timer4)
  ; Bit 6: UART2 timer (0 = Timer 3, 1 = Timer4)
  ; Bit 5: UART1 timer (0 = Timer 3, 1 = Timer4)
  ; Bit 4: UART1 timer (0 = Timer 3, 1 = Timer4)
  ; Bit 3-2: PWM Edge Interrupt Select on P62
  ;          (0=disable, 1=positive edge, 2=negative edge, 3=both edges)
  ; Bit 1: Timer 4 Output Enable (1=enable, 0=disable)
  ; Bit 0: Timer 4 Input clock (0=FCLK, 1=P60)
  lda.b #0x88
  sta TCR

  ; Enable timer 2, disable timer 0
  lda TER
  ora.b #0x04
  and.b #0xfe
  sta TER

  ; Disable UART, Edge, and Parallel Interface Interrupt
  lda.b #0x00
  sta UIER
  sta PIBER
  sta EIER

  ; UART3 control: receiver enable, 8 bit, transmit enable
  lda.b #0x25
  sta ACSR3

  ; Set port PD50-53 as output (for possible future project.. only P50
  ; is used here.
  lda.b #0x0f
  sta PDD5

  ; set port 2 value to 0
  lda.b #0
  sta PD5

  ; Set timer top value
  ; (3686400 / 16 / 12) = 19200
  ; 1 / 19200 = 0.000052 seconds - approx 50us per interrupt
  ; 16 * 12 cpu cycles per interrupt
  ldx #12
  stx T2LL

  ; Set timer interrupt vector.  These RAM addresses are pointed to
  ; from the ROM address of IRQT2 (0xff84, 0xff85).  When an interrupt
  ; occurs, the CPU will jump to this area (which has a jump instruction
  ; for the address in 0x0121, 0x0122.
  lda.b #timer2_interrupt & 0xff
  sta 0x0121
  lda.b #timer2_interrupt >> 8
  sta 0x0122
  lda.b #0
  sta 0x0123

  ; Setup servo timer (not really needed here)
  SET_SERVO(servo_count_0, HIPPO_REST_0,interrupt_count_0)
  SET_SERVO(servo_count_1, HIPPO_REST_1,interrupt_count_1)
  SET_SERVO(servo_count_2, HIPPO_REST_2,interrupt_count_2)
  SET_SERVO(servo_count_3, HIPPO_REST_3,interrupt_count_3)

  ldx #0
  stx interrupt_count_0
  stx interrupt_count_1
  stx interrupt_count_2
  stx interrupt_count_3

  ldx #TOP_SERVO_COUNTER
  stx delay

  ; Enable interrupt for Timer 2
  lda.b #0x04
  sta TIER

  ; Enable interrupts
  cli

  ; Disable interrupt for Timer 2
  ;sep #0x20
  ;lda.b #0x00
  ;sta TIER

main:
  ldx interrupt_count_0
  cpx #0
  bne ignore_0
  SET_SERVO_REST(servo_count_0, HIPPO_REST_0)
ignore_0:

  ldx interrupt_count_1
  cpx #0
  bne ignore_1
  SET_SERVO_REST(servo_count_1, HIPPO_REST_1)
ignore_1:

  ldx interrupt_count_2
  cpx #0
  bne ignore_2
  SET_SERVO_REST(servo_count_2, HIPPO_REST_2)
ignore_2:

  ldx interrupt_count_3
  cpx #0
  bne ignore_3
  SET_SERVO_REST(servo_count_3, HIPPO_REST_3)
ignore_3:

  ; Test if UART3 receive interrupt flag is set
  lda UIFR
  bit.b #0x40
  beq main

  ; Clear UART interrupt flag
  lda.b #0x00
  sta UIFR

  lda ARTD3
  sta ARTD3

  cmp.b #'0'
  bne not_zero
  SET_SERVO(servo_count_0, HIPPO_PULL_0, interrupt_count_0)
  jmp main
not_zero:

  cmp.b #'1'
  bne not_one
  SET_SERVO(servo_count_1, HIPPO_PULL_1, interrupt_count_1)
  jmp main
not_one:

  cmp.b #'2'
  bne not_two
  SET_SERVO(servo_count_2, HIPPO_PULL_2, interrupt_count_2)
  jmp main
not_two:

  cmp.b #'3'
  bne not_three
  SET_SERVO(servo_count_3, HIPPO_PULL_3, interrupt_count_3)
  jmp main
not_three:
  jmp main

.align 16
timer2_interrupt:
  ; Save A and status register (from main context)
  pha
  php

  ; Increment interrupt_count
  ;ldy interrupt_count
  ;iny
  ;sty interrupt_count

  ; Set A to 16-bit
  rep #0x20
  dec interrupt_count_0
  dec interrupt_count_1
  dec interrupt_count_2
  dec interrupt_count_3
  sep #0x20

  ; Clear interrupt for Timer 2
  lda.b #0x04
  sta TIFR

  ; Update delay count
  ldy delay
  dey
  sty delay
  bne timer2_interrupt_not_0

  ; Reset delay count
  ldy #TOP_SERVO_COUNTER
  sty delay

  ; Set P50-P53 on
  ;lda PD5
  ;ora.b #0x0f
  lda.b #0x0f
  sta PD5

  lda.b #0xff
  sta PD7

  ; Restore A and status register
  plp
  pla
  rti

timer2_interrupt_not_0:
  ;cpy servo_count_0
  ;bne timer2_do_not_update_servo_0
  ;lda PD5
  ;and.b #0xfe
  ;sta PD5
;timer2_do_not_update_servo_0:
  CHECK_SERVO(servo_count_0, 0xfe)
  CHECK_SERVO(servo_count_1, 0xfd)
  CHECK_SERVO(servo_count_2, 0xfb)
  CHECK_SERVO(servo_count_3, 0xf7)

timer2_interrupt_exit:
  ; Restore A and status register
  plp
  pla
  rti

